#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/05/08
#Last Modify: 2018/05/08
# deeper learning insights machine learning page 638

# Consider a set of eigenvalues as indicative of how much "new information
# content" our features bring to the dataset; a low eigenvalue suggests that
# the data may be correlated with other features. For an example of this at
# work, consider the following code, which creates a feature set and then adds
# collinearity to features 0, 2, and 4:

import numpy as np
x = np.random.randn(100, 5)
noise = np.random.randn(100)
x[:,4] = 2 * x[:,0] + 3 * x[:,2] + .5 * noise
# When we generate the correlation matrix and compute eigenvalues, we find the
# following:
corr = np.corrcoef(x, rowvar=0)
w, v = np.linalg.eig(corr)
print('eigenvalues of features in the dataset x')
print(w)
# eigenvalues of features in the dataset x
# [ 0.00716428 1.94474029 1.30385565 0.74699492
#  0.99724486]
# Clearly, our 0th feature is suspect! We can then inspect the eigenvalues of
# this feature via calling v:
print('eigenvalues of eigenvector 0')
print(v[:,0])
# eigenvalues of eigenvector 0
# [-0.35663659 -0.00853105 -0.62463305
#  0.00959048
#  0.69460718]
# From the small values of features in position one and three, we can tell that
# features 2 and 4 are highly multicollinear with feature 0. We ought to remove
# two of these three features before proceeding!

